from typing import overload
import typing

import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.Storage
import QuantConnect.Packets
import System
import System.Collections
import System.Collections.Generic

QuantConnect_Lean_Engine_Storage__EventContainer_Callable = typing.TypeVar("QuantConnect_Lean_Engine_Storage__EventContainer_Callable")
QuantConnect_Lean_Engine_Storage__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Lean_Engine_Storage__EventContainer_ReturnType")


class StorageLimitExceededException(System.Exception):
    """Exception thrown when the object store storage limit has been exceeded"""

    def __init__(self, message: str) -> None:
        """
        Creates a new instance of the storage limit exceeded exception
        
        :param message: The associated message
        """
        ...


class LocalObjectStore(System.Object, QuantConnect.Interfaces.IObjectStore, typing.Iterable[System.Collections.Generic.KeyValuePair[str, typing.List[int]]]):
    """A local disk implementation of IObjectStore."""

    NoReadPermissionsError: str = ...
    """
    No read permissions error message
    
    This field is protected.
    """

    NoWritePermissionsError: str = ...
    """
    No write permissions error message
    
    This field is protected.
    """

    @property
    def ErrorRaised(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Interfaces.ObjectStoreErrorRaisedEventArgs], None], None]:
        """Event raised each time there's an error"""
        ...

    @ErrorRaised.setter
    def ErrorRaised(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Interfaces.ObjectStoreErrorRaisedEventArgs], None], None]):
        """Event raised each time there's an error"""
        ...

    DefaultObjectStore: str
    """Gets the default object store location"""

    @property
    def Controls(self) -> QuantConnect.Packets.Controls:
        """
        Provides access to the controls governing behavior of this instance, such as the persistence interval
        
        This property is protected.
        """
        ...

    @Controls.setter
    def Controls(self, value: QuantConnect.Packets.Controls):
        """
        Provides access to the controls governing behavior of this instance, such as the persistence interval
        
        This property is protected.
        """
        ...

    @property
    def AlgorithmStorageRoot(self) -> str:
        """
        The root storage folder for the algorithm
        
        This property is protected.
        """
        ...

    @AlgorithmStorageRoot.setter
    def AlgorithmStorageRoot(self, value: str):
        """
        The root storage folder for the algorithm
        
        This property is protected.
        """
        ...

    def ContainsKey(self, key: str) -> bool:
        """
        Determines whether the store contains data for the specified key
        
        :param key: The object key
        :returns: True if the key was found.
        """
        ...

    def Delete(self, key: str) -> bool:
        """
        Deletes the object data for the specified key
        
        :param key: The object key
        :returns: True if the delete operation was successful.
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System.Collections.Generic.KeyValuePair[str, typing.List[int]]]:
        """
        Returns an enumerator that iterates through the collection.
        
        :returns: A System.Collections.Generic.IEnumerator`1 that can be used to iterate through the collection.
        """
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.IEnumerator:
        """
        Returns an enumerator that iterates through a collection.
        
        :returns: An System.Collections.IEnumerator object that can be used to iterate through the collection.
        """
        ...

    def GetFilePath(self, key: str) -> str:
        """
        Returns the file path for the specified key
        
        :param key: The object key
        :returns: The path for the file.
        """
        ...

    def Initialize(self, algorithmName: str, userId: int, projectId: int, userToken: str, controls: QuantConnect.Packets.Controls) -> None:
        """
        Initializes the object store
        
        :param algorithmName: The algorithm name
        :param userId: The user id
        :param projectId: The project id
        :param userToken: The user token
        :param controls: The job controls instance
        """
        ...

    def InternalSaveBytes(self, key: str, contents: typing.List[int]) -> bool:
        """
        Won't trigger persist nor will check storage write permissions, useful on initialization since it allows read only permissions to load the object store
        
        This method is protected.
        """
        ...

    def LoadExistingObjects(self) -> None:
        """
        Loads objects from the AlgorithmStorageRoot into the ObjectStore
        
        This method is protected.
        """
        ...

    def OnErrorRaised(self, error: System.Exception) -> None:
        """
        Event invocator for the ErrorRaised event
        
        This method is protected.
        """
        ...

    def PathForKey(self, key: str) -> str:
        """
        Get's a file path for a given key.
        Internal use only because it does not guarantee the existence of the file.
        
        This method is protected.
        """
        ...

    def PersistData(self, data: System.Collections.Generic.IEnumerable[System.Collections.Generic.KeyValuePair[str, typing.List[int]]]) -> bool:
        """
        Overridable persistence function
        
        This method is protected.
        
        :param data: The data to be persisted
        :returns: True if persistence was successful, otherwise false.
        """
        ...

    def ReadBytes(self, key: str) -> typing.List[int]:
        """
        Returns the object data for the specified key
        
        :param key: The object key
        :returns: A byte array containing the data.
        """
        ...

    def SaveBytes(self, key: str, contents: typing.List[int]) -> bool:
        """
        Saves the object data for the specified key
        
        :param key: The object key
        :param contents: The object data
        :returns: True if the save operation was successful.
        """
        ...


class _EventContainer(typing.Generic[QuantConnect_Lean_Engine_Storage__EventContainer_Callable, QuantConnect_Lean_Engine_Storage__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Lean_Engine_Storage__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Lean_Engine_Storage__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Lean_Engine_Storage__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
import QuantConnect.Securities.FutureOption.Api
import System
import System.Collections.Generic


class CMEOptionExpirationEntry(System.Object):
    """Chicago Mercantile Exchange Option Expiration Entry"""

    @property
    def Month(self) -> int:
        """Month of expiry"""
        ...

    @Month.setter
    def Month(self, value: int):
        """Month of expiry"""
        ...

    @property
    def Year(self) -> int:
        """Year of expiry"""
        ...

    @Year.setter
    def Year(self, value: int):
        """Year of expiry"""
        ...

    @property
    def Code(self) -> str:
        """Expiration code (two letter)"""
        ...

    @Code.setter
    def Code(self, value: str):
        """Expiration code (two letter)"""
        ...

    @property
    def TwoDigitsCode(self) -> str:
        """Expiration code (three letter)"""
        ...

    @TwoDigitsCode.setter
    def TwoDigitsCode(self, value: str):
        """Expiration code (three letter)"""
        ...


class CMEOptionsExpiration(System.Object):
    """
    Future options Expiration entries. These are useful because we can derive the
    future chain from this data, since FOP and FUT share a 1-1 expiry code.
    """

    @property
    def Label(self) -> str:
        """Date of expiry"""
        ...

    @Label.setter
    def Label(self, value: str):
        """Date of expiry"""
        ...

    @property
    def ProductId(self) -> int:
        """Product ID of the expiring asset (usually future option)"""
        ...

    @ProductId.setter
    def ProductId(self, value: int):
        """Product ID of the expiring asset (usually future option)"""
        ...

    @property
    def ContractId(self) -> str:
        """Contract ID of the asset"""
        ...

    @ContractId.setter
    def ContractId(self, value: str):
        """Contract ID of the asset"""
        ...

    @property
    def Expiration(self) -> QuantConnect.Securities.FutureOption.Api.CMEOptionExpirationEntry:
        """Contract month code formatted as [FUTURE_MONTH_LETTER(1)][YEAR(1)]"""
        ...

    @Expiration.setter
    def Expiration(self, value: QuantConnect.Securities.FutureOption.Api.CMEOptionExpirationEntry):
        """Contract month code formatted as [FUTURE_MONTH_LETTER(1)][YEAR(1)]"""
        ...


class CMEOptionsTradeDatesAndExpiration(System.Object):
    """CME options trades, dates, and expiration list API call root response"""

    @property
    def Label(self) -> str:
        """Describes the type of future option this entry is"""
        ...

    @Label.setter
    def Label(self, value: str):
        """Describes the type of future option this entry is"""
        ...

    @property
    def Name(self) -> str:
        """Name of the product"""
        ...

    @Name.setter
    def Name(self, value: str):
        """Name of the product"""
        ...

    @property
    def OptionType(self) -> str:
        """
        Option type. "AME" for American, "EUR" for European.
        Note that there are other types such as weekly, but we
        only support American options for now.
        """
        ...

    @OptionType.setter
    def OptionType(self, value: str):
        """
        Option type. "AME" for American, "EUR" for European.
        Note that there are other types such as weekly, but we
        only support American options for now.
        """
        ...

    @property
    def ProductId(self) -> int:
        """Product ID of the option"""
        ...

    @ProductId.setter
    def ProductId(self, value: int):
        """Product ID of the option"""
        ...

    @property
    def Daily(self) -> bool:
        """Is Daily option"""
        ...

    @Daily.setter
    def Daily(self, value: bool):
        """Is Daily option"""
        ...

    @property
    def Sto(self) -> bool:
        """???"""
        ...

    @Sto.setter
    def Sto(self, value: bool):
        """???"""
        ...

    @property
    def Weekly(self) -> bool:
        """Is weekly option"""
        ...

    @Weekly.setter
    def Weekly(self, value: bool):
        """Is weekly option"""
        ...

    @property
    def Expirations(self) -> System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEOptionsExpiration]:
        """Expirations of the future option"""
        ...

    @Expirations.setter
    def Expirations(self, value: System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEOptionsExpiration]):
        """Expirations of the future option"""
        ...


class CMEOptionChainQuoteEntry(System.Object):
    """Option chain entry quotes, containing strike price"""

    @property
    def StrikePrice(self) -> float:
        """Strike price of the future option quote entry"""
        ...

    @StrikePrice.setter
    def StrikePrice(self, value: float):
        """Strike price of the future option quote entry"""
        ...


class CMEOptionChainQuotes(System.Object):
    """CME Option Chain Quotes API call root response"""

    @property
    def Quotes(self) -> System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEOptionChainQuoteEntry]:
        """The future options contracts with/without settlements"""
        ...

    @Quotes.setter
    def Quotes(self, value: System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEOptionChainQuoteEntry]):
        """The future options contracts with/without settlements"""
        ...


class CMEProductSlateV2ListEntry(System.Object):
    """Product entry describing the asset matching the search criteria"""

    @property
    def Id(self) -> int:
        """CME ID for the asset"""
        ...

    @Id.setter
    def Id(self, value: int):
        """CME ID for the asset"""
        ...

    @property
    def Name(self) -> str:
        """Name of the product (e.g. E-mini NASDAQ futures)"""
        ...

    @Name.setter
    def Name(self, value: str):
        """Name of the product (e.g. E-mini NASDAQ futures)"""
        ...

    @property
    def Clearing(self) -> str:
        """Clearing code"""
        ...

    @Clearing.setter
    def Clearing(self, value: str):
        """Clearing code"""
        ...

    @property
    def Globex(self) -> str:
        """GLOBEX ticker"""
        ...

    @Globex.setter
    def Globex(self, value: str):
        """GLOBEX ticker"""
        ...

    @property
    def GlobexTraded(self) -> bool:
        """Is traded in the GLOBEX venue"""
        ...

    @GlobexTraded.setter
    def GlobexTraded(self, value: bool):
        """Is traded in the GLOBEX venue"""
        ...

    @property
    def Venues(self) -> str:
        """Venues this asset trades on"""
        ...

    @Venues.setter
    def Venues(self, value: str):
        """Venues this asset trades on"""
        ...

    @property
    def Cleared(self) -> str:
        """Asset type this product is cleared as (i.e. "Futures", "Options")"""
        ...

    @Cleared.setter
    def Cleared(self, value: str):
        """Asset type this product is cleared as (i.e. "Futures", "Options")"""
        ...

    @property
    def Exchange(self) -> str:
        """Exchange the asset trades on (i.e. CME, NYMEX, COMEX, CBOT)"""
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        """Exchange the asset trades on (i.e. CME, NYMEX, COMEX, CBOT)"""
        ...

    @property
    def GroupId(self) -> int:
        """Asset class group ID - describes group of asset class (e.g. equities, agriculture, etc.)"""
        ...

    @GroupId.setter
    def GroupId(self, value: int):
        """Asset class group ID - describes group of asset class (e.g. equities, agriculture, etc.)"""
        ...

    @property
    def subGroupId(self) -> int:
        """More specific ID describing product"""
        ...

    @subGroupId.setter
    def subGroupId(self, value: int):
        """More specific ID describing product"""
        ...


class CMEProductSlateV2ListResponse(System.Object):
    """Product slate API call root response"""

    @property
    def Products(self) -> System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEProductSlateV2ListEntry]:
        """Products matching the search criteria"""
        ...

    @Products.setter
    def Products(self, value: System.Collections.Generic.List[QuantConnect.Securities.FutureOption.Api.CMEProductSlateV2ListEntry]):
        """Products matching the search criteria"""
        ...





from typing import overload
import QuantConnect.Algorithm.Framework.Alphas
import QuantConnect.Algorithm.Framework.Alphas.Analysis
import QuantConnect.Algorithm.Framework.Alphas.Analysis.Functions
import System


class BinaryInsightScoreFunction(System.Object, QuantConnect.Algorithm.Framework.Alphas.Analysis.IInsightScoreFunction):
    """
    Defines a scoring function that always returns 1 or 0.
    You're either right or you're wrong with this one :)
    """

    def Evaluate(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext, scoreType: QuantConnect.Algorithm.Framework.Alphas.InsightScoreType) -> float:
        ...


